/*
Implementation of the "Redirect" class + SWIG/Manual wrappers.
The Rubyembed Project
$Id: redirect.cpp,v 1.4 2003/04/17 02:41:37 neoneye Exp $
*/
#include <iostream>
#include "redirect.h"
#include "rubycpp.h"

using std::cout;
using std::endl;

Redirect::Redirect() { 
	this->view = Redirect::global_view;
	cout << "redirect.ctor: hello" << endl;
}

Redirect::~Redirect() {
	cout << "redirect.dtor: hello" << endl;
}

void Redirect::repaint() {
	RUBY_TRY {
		cout << "redirect.repaint: enter" << endl;
		view->repaint();
		cout << "redirect.repaint: leave" << endl;
	} RUBY_CATCH
}

void Redirect::SetView(EMBEDRUBY::View *parent) {
	Redirect::global_view = parent;
}

EMBEDRUBY::View *Redirect::global_view = 0;


#if EMBEDRUBY_SWIG != 0
/* -----------------------------------
SWIG has generated the necessary 
wrapper code for us. All we have is
just to call Init_Embed();
----------------------------------- */

extern "C" void Init_Embed(); 

void Redirect::RubyInit() {
	Init_Embed();
}

#else
/* -----------------------------------
This is how it looks if you are 
writing the wrapper yourself.
As you can imagine this can quickly
become messy and confusing.
----------------------------------- */

#include <ruby.h>

namespace WRAP {

void Free(Redirect *p) {
	delete p;
}

VALUE Alloc(VALUE self) {
	Redirect* p = new Redirect();
	return Data_Wrap_Struct(self, 0, Free, p);
}

VALUE Repaint(VALUE self) {
	Redirect *p;
	Data_Get_Struct(self, Redirect, p);
	p->repaint();
	return self;
}

} // end of namespace WRAP

void Redirect::RubyInit() {
	VALUE module = rb_define_module("Embed");
	VALUE h = rb_define_class_under(module, "Redirect", rb_cObject);
	rb_define_alloc_func(h, WRAP::Alloc); 
	typedef VALUE (*HOOK)(...);
	rb_define_method(h, "repaint", reinterpret_cast<HOOK>(&WRAP::Repaint), 0);
}

#endif
